@@ -12,7 +12,8 @@ import re
 from typing import Any, Callable, Dict, List, Tuple, Union
 
 from filefinder.matcher import (Matcher, Matches,
-                                get_matchers_indices)
+                                get_matchers_indices,
+                                InvalidMatcher)
 
 log = logging.getLogger(__name__)
 
@@ -323,6 +324,8 @@ class Finder():
         segments = self.segments.copy()
 
         for idx, value in fixed_matchers.items():
+            if isinstance(value, bool):
+                value = self.matchers[idx].opt[value]
             if isinstance(value, (list, tuple)):
                 value = value[0]
             if not isinstance(value, str):
@@ -403,17 +406,43 @@ class Finder():
         Add matchers objects to self.
         Set segments attribute.
         """
-        splits = [0]
+        matchers_starts = [m.start()+1
+                           for m in re.finditer(r'%\(', self.pregex)]
+
         self.matchers = []
-        for i, m in enumerate(re.finditer(Matcher.REGEX, self.pregex)):
-            self.matchers.append(Matcher(m, i))
-            splits += [m.start(), m.end()]
+        splits = [0]
+        for idx, start in enumerate(matchers_starts):
+            end = None
+            level = 1
+            for i, c in enumerate(self.pregex[start+1:]):
+                if c == "(":
+                    level += 1
+                elif c == ")":
+                    level -= 1
+                    if level == 0:
+                        end = start+i+1
+                        break
+
+            if end is None:
+                end = start+6
+                substr = self.pregex[start-1:end]
+                if end < len(self.pregex):
+                    substr += '...'
+                raise ValueError("No matcher end found for '{}'"
+                                 .format(substr))
+
+            try:
+                self.matchers.append(Matcher(self.pregex[start+1:end], idx))
+                splits += [start-1, end+1]  # -1 removes the %
+            except InvalidMatcher:
+                pass
+
         self.segments = [self.pregex[i:j]
                          for i, j in zip(splits, splits[1:]+[None])]
 
         # Replace matcher by its regex
         for idx, m in enumerate(self.matchers):
-            self.segments[2*idx+1] = '({})'.format(m.get_regex())
+            self.segments[2*idx+1] = m.get_regex()
 
     def update_regex(self):
         """Update regex.
@@ -427,6 +456,9 @@ class Finder():
                 segments[i] = s if i % 2 == 1 else re.escape(s)
 
         for idx, value in self.fixed_matchers.items():
+            if isinstance(value, bool):
+                if isinstance(self.matchers[idx].opt, tuple):
+                    value = self.matchers[idx].opt[value]
             if not isinstance(value, (list, tuple)):
                 value = [value]
             value = [v if isinstance(v, str)
